==> ui-18.1.0.tgz: package/components/upload/Upload.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isNull, isNumber } from 'lodash';
import React, { useRef } from 'react';
import { useForkRef, useUnmount } from '@react-devui/hooks';
import { getClassName, getUID } from '@react-devui/utils';
import { useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DList } from './List';
import { DPicture } from './Picture';
import { DPictureList } from './PictureList';
import { DUploadAction } from './UploadAction';
import { DUploadPictureButton } from './UploadPictureButton';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DUpload' });
function Upload(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dFormControl, dModel, dXHRRequest, dListType = 'list', dDrag = false, dMax, dDefaultActions, dActions, dCustomUpload, dBeforeUpload, onModelChange, onRemove } = _b, restProps = __rest(_b, ["children", "dFormControl", "dModel", "dXHRRequest", "dListType", "dDrag", "dMax", "dDefaultActions", "dActions", "dCustomUpload", "dBeforeUpload", "onModelChange", "onRemove"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const inputRef = useRef(null);
    const combineInputRef = useForkRef(inputRef, ref);
    //#endregion
    const dataRef = useRef({
        fileURLs: [],
    });
    const formControlInject = useFormControl(dFormControl);
    const [fileList, changeFileList] = useDValue([], dModel, undefined, undefined, formControlInject);
    useUnmount(() => {
        dataRef.current.fileURLs.forEach((fileURL) => {
            URL.revokeObjectURL(fileURL);
        });
    });
    const handleFiles = (files) => {
        if (dCustomUpload) {
            dCustomUpload(files);
        }
        else {
            const addList = [];
            for (let index = 0; index < files.length; index++) {
                const file = files.item(index);
                if (file) {
                    const xhr = new XMLHttpRequest();
                    let uid = getUID();
                    const add = (obj) => {
                        const fileURL = URL.createObjectURL(file);
                        dataRef.current.fileURLs.push(fileURL);
                        const addFile = Object.assign({ uid, url: fileURL, thumbUrl: file.type.startsWith('image') ? fileURL : undefined, name: file.name, originFile: file, response: xhr.response }, obj);
                        if (isNumber(dMax) && fileList.length + addList.length >= dMax) {
                            if (dMax === 1) {
                                uid = addFile.uid = fileList[0].uid;
                                changeFileList([addFile]);
                                onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange([addFile], {
                                    type: 'update',
                                    files: [addFile],
                                });
                                return;
                            }
                            return true;
                        }
                        addList.push(addFile);
                    };
                    const update = (obj) => {
                        let hasChange = true;
                        const newList = changeFileList((draft) => {
                            const index = draft.findIndex((f) => f.uid === uid);
                            if (index !== -1) {
                                draft[index] = Object.assign(draft[index], Object.assign({ response: xhr.response }, obj));
                            }
                            else {
                                hasChange = false;
                            }
                        });
                        if (hasChange) {
                            onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, {
                                type: 'update',
                                files: [newList.find((f) => f.uid === uid)],
                            });
                        }
                    };
                    const upload = (condition) => {
                        if (condition === false) {
                            add({ status: null });
                        }
                        else {
                            const abort = add({ status: 'progress', percent: 0 });
                            if (abort) {
                                return;
                            }
                            xhr.upload.addEventListener('progress', (e) => {
                                if (e.lengthComputable) {
                                    const percent = Math.round((e.loaded * 100) / e.total);
                                    update({ status: 'progress', percent });
                                }
                            });
                            xhr.addEventListener('error', () => {
                                update({ status: 'error', percent: undefined });
                            });
                            xhr.addEventListener('load', () => {
                                update({ status: xhr.status >= 200 && xhr.status < 300 ? 'load' : 'error', percent: undefined });
                            });
                            const { method = 'POST', url, responseType = 'json', header = {}, body = (file) => {
                                const formData = new FormData();
                                formData.append('file', file);
                                return formData;
                            }, custom, } = dXHRRequest;
                            xhr.open(method, url);
                            xhr.responseType = responseType;
                            Object.entries(header).forEach((h) => {
                                xhr.setRequestHeader(...h);
                            });
                            custom === null || custom === void 0 ? void 0 : custom(xhr);
                            xhr.send(body(condition === true ? file : condition));
                        }
                    };
                    let shouldUpload = true;
                    if (dBeforeUpload) {
                        shouldUpload = dBeforeUpload(file, files);
                        if (shouldUpload instanceof Promise) {
                            shouldUpload.then((condition) => {
                                upload(condition);
                            });
                        }
                        else {
                            upload(shouldUpload);
                        }
                    }
                    else {
                        upload(true);
                    }
                }
            }
            if (addList.length > 0) {
                const newList = changeFileList((draft) => draft.concat(addList));
                onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, {
                    type: 'add',
                    files: addList,
                });
            }
        }
    };
    const child = (() => {
        if (isNull(children)) {
            return children;
        }
        let dragProps = {};
        if (dDrag) {
            dragProps = {
                onDragEnter: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onDragEnter) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    e.stopPropagation();
                    e.preventDefault();
                },
                onDragOver: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onDragOver) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    e.stopPropagation();
                    e.preventDefault();
                },
                onDrop: (e) => {
                    var _a, _b;
                    (_b = (_a = children.props).onDrop) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    e.stopPropagation();
                    e.preventDefault();
                    const dt = e.dataTransfer;
                    const files = dt.files;
                    handleFiles(files);
                },
            };
        }
        return React.cloneElement(children, Object.assign(Object.assign({}, dragProps), { onClick: (e) => {
                var _a, _b;
                (_b = (_a = children.props).onClick) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                if (inputRef.current) {
                    inputRef.current.click();
                }
            } }));
    })();
    const listProps = {
        dFileList: fileList,
        dDefaultActions,
        onRemove: (file) => {
            onRemove === null || onRemove === void 0 ? void 0 : onRemove(file);
            const newList = changeFileList((draft) => {
                const index = draft.findIndex((f) => f.uid === file.uid);
                draft.splice(index, 1);
            });
            onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, { type: 'remove', files: [file] });
        },
    };
    return (_jsxs(_Fragment, { children: [_jsx("input", Object.assign({}, restProps, { ref: combineInputRef, className: getClassName(restProps.className, `${dPrefix}upload`), type: (_a = restProps.type) !== null && _a !== void 0 ? _a : 'file', onChange: (e) => {
                    var _a;
                    (_a = restProps.onChange) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                    const files = e.currentTarget.files;
                    if (files) {
                        handleFiles(files);
                    }
                    e.currentTarget.value = '';
                } })), dListType === 'list' ? (_jsxs(_Fragment, { children: [child, _jsx(DList, Object.assign({}, listProps, { dActions: dActions !== null && dActions !== void 0 ? dActions : (() => [_jsx(DUploadAction, { dPreset: "remove" })]) }))] })) : dListType === 'picture' ? (_jsx(DPicture, Object.assign({}, listProps, { dActions: dActions !== null && dActions !== void 0 ? dActions : (() => [_jsx(DUploadAction, { dPreset: "preview" }), _jsx(DUploadAction, { dPreset: "remove" })]) }, { children: child }))) : dListType === 'picture-list' ? (_jsxs(_Fragment, { children: [child, _jsx(DPictureList, Object.assign({}, listProps, { dActions: dActions !== null && dActions !== void 0 ? dActions : (() => [_jsx(DUploadAction, { dPreset: "remove" })]) }))] })) : (child)] }));
}
export const DUpload = React.forwardRef(Upload);
DUpload.Action = DUploadAction;
DUpload.PictureButton = DUploadPictureButton;

==> ui-18.3.4.tgz: package/components/upload/Upload.js <==
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
import { jsx as _jsx, Fragment as _Fragment, jsxs as _jsxs } from "react/jsx-runtime";
import { isNumber } from 'lodash';
import React, { useRef } from 'react';
import { useForkRef, useUnmount } from '@react-devui/hooks';
import { getClassName, getUID } from '@react-devui/utils';
import { useDValue } from '../../hooks';
import { registerComponentMate } from '../../utils';
import { useFormControl } from '../form';
import { useComponentConfig, usePrefixConfig } from '../root';
import { DUploadAction } from './UploadAction';
import { DUploadList } from './UploadList';
import { DUploadPictureButton } from './UploadPictureButton';
const { COMPONENT_NAME } = registerComponentMate({ COMPONENT_NAME: 'DUpload' });
function Upload(props, ref) {
    var _a;
    const _b = useComponentConfig(COMPONENT_NAME, props), { children, dFormControl, dModel, dTrigger, dList, dXHRRequest, dDrag = false, dMax, dCustomUpload, dBeforeUpload, onModelChange, onRemove } = _b, restProps = __rest(_b, ["children", "dFormControl", "dModel", "dTrigger", "dList", "dXHRRequest", "dDrag", "dMax", "dCustomUpload", "dBeforeUpload", "onModelChange", "onRemove"]);
    //#region Context
    const dPrefix = usePrefixConfig();
    //#endregion
    //#region Ref
    const inputRef = useRef(null);
    const combineInputRef = useForkRef(inputRef, ref);
    //#endregion
    const dataRef = useRef({
        fileURLs: [],
    });
    const formControlInject = useFormControl(dFormControl);
    const [fileList, changeFileList] = useDValue([], dModel, undefined, undefined, formControlInject);
    useUnmount(() => {
        dataRef.current.fileURLs.forEach((fileURL) => {
            URL.revokeObjectURL(fileURL);
        });
    });
    const handleFiles = (files) => {
        if (dCustomUpload) {
            dCustomUpload(files);
        }
        else {
            const addList = [];
            for (let index = 0; index < files.length; index++) {
                const file = files.item(index);
                if (file) {
                    const xhr = new XMLHttpRequest();
                    let uid = getUID();
                    const add = (obj) => {
                        const fileURL = URL.createObjectURL(file);
                        dataRef.current.fileURLs.push(fileURL);
                        const addFile = Object.assign({ uid, url: fileURL, thumbUrl: file.type.startsWith('image') ? fileURL : undefined, name: file.name, originFile: file, response: xhr.response }, obj);
                        if (isNumber(dMax) && fileList.length + addList.length >= dMax) {
                            if (dMax === 1) {
                                uid = addFile.uid = fileList[0].uid;
                                changeFileList([addFile]);
                                onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange([addFile], {
                                    type: 'update',
                                    files: [addFile],
                                });
                                return;
                            }
                            return true;
                        }
                        addList.push(addFile);
                    };
                    const update = (obj) => {
                        let hasChange = true;
                        const newList = changeFileList((draft) => {
                            const index = draft.findIndex((f) => f.uid === uid);
                            if (index !== -1) {
                                draft[index] = Object.assign(draft[index], Object.assign({ response: xhr.response }, obj));
                            }
                            else {
                                hasChange = false;
                            }
                        });
                        if (hasChange) {
                            onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, {
                                type: 'update',
                                files: [newList.find((f) => f.uid === uid)],
                            });
                        }
                    };
                    const upload = (condition) => {
                        if (condition === false) {
                            add({ status: null });
                        }
                        else {
                            const abort = add({ status: 'progress', percent: 0 });
                            if (abort) {
                                return;
                            }
                            xhr.upload.addEventListener('progress', (e) => {
                                if (e.lengthComputable) {
                                    const percent = Math.round((e.loaded * 100) / e.total);
                                    update({ status: 'progress', percent });
                                }
                            });
                            xhr.addEventListener('error', () => {
                                update({ status: 'error', percent: undefined });
                            });
                            xhr.addEventListener('load', () => {
                                update({ status: xhr.status >= 200 && xhr.status < 300 ? 'load' : 'error', percent: undefined });
                            });
                            const { method = 'POST', url, responseType = 'json', header = {}, body = (file) => {
                                const formData = new FormData();
                                formData.append('file', file);
                                return formData;
                            }, custom, } = dXHRRequest;
                            xhr.open(method, url);
                            xhr.responseType = responseType;
                            Object.entries(header).forEach((h) => {
                                xhr.setRequestHeader(...h);
                            });
                            custom === null || custom === void 0 ? void 0 : custom(xhr);
                            xhr.send(body(condition === true ? file : condition));
                        }
                    };
                    let shouldUpload = true;
                    if (dBeforeUpload) {
                        shouldUpload = dBeforeUpload(file, files);
                        if (shouldUpload instanceof Promise) {
                            shouldUpload.then((condition) => {
                                upload(condition);
                            });
                        }
                        else {
                            upload(shouldUpload);
                        }
                    }
                    else {
                        upload(true);
                    }
                }
            }
            if (addList.length > 0) {
                const newList = changeFileList((draft) => draft.concat(addList));
                onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, {
                    type: 'add',
                    files: addList,
                });
            }
        }
    };
    const trigger = (() => {
        if (dTrigger) {
            let dragProps = {};
            if (dDrag) {
                dragProps = {
                    onDragEnter: (e) => {
                        var _a, _b;
                        (_b = (_a = dTrigger.props).onDragEnter) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        e.stopPropagation();
                        e.preventDefault();
                    },
                    onDragOver: (e) => {
                        var _a, _b;
                        (_b = (_a = dTrigger.props).onDragOver) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        e.stopPropagation();
                        e.preventDefault();
                    },
                    onDrop: (e) => {
                        var _a, _b;
                        (_b = (_a = dTrigger.props).onDrop) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                        e.stopPropagation();
                        e.preventDefault();
                        const dt = e.dataTransfer;
                        const files = dt.files;
                        handleFiles(files);
                    },
                };
            }
            return React.cloneElement(dTrigger, Object.assign(Object.assign({}, dragProps), { onClick: (e) => {
                    var _a, _b;
                    (_b = (_a = dTrigger.props).onClick) === null || _b === void 0 ? void 0 : _b.call(_a, e);
                    if (inputRef.current) {
                        inputRef.current.click();
                    }
                } }));
        }
    })();
    return (_jsxs(_Fragment, { children: [_jsx("input", Object.assign({}, restProps, { ref: combineInputRef, className: getClassName(restProps.className, `${dPrefix}upload`), type: (_a = restProps.type) !== null && _a !== void 0 ? _a : 'file', onChange: (e) => {
                    var _a;
                    (_a = restProps.onChange) === null || _a === void 0 ? void 0 : _a.call(restProps, e);
                    const files = e.currentTarget.files;
                    if (files) {
                        handleFiles(files);
                    }
                    e.currentTarget.value = '';
                } })), children === null || children === void 0 ? void 0 : children({
                trigger,
                list: dList &&
                    React.cloneElement(dList, {
                        __fileList: fileList,
                        __onRemove(file) {
                            onRemove === null || onRemove === void 0 ? void 0 : onRemove(file);
                            const newList = changeFileList((draft) => {
                                const index = draft.findIndex((f) => f.uid === file.uid);
                                draft.splice(index, 1);
                            });
                            onModelChange === null || onModelChange === void 0 ? void 0 : onModelChange(newList, { type: 'remove', files: [file] });
                        },
                    }),
            })] }));
}
export const DUpload = React.forwardRef(Upload);
DUpload.Action = DUploadAction;
DUpload.List = DUploadList;
DUpload.PictureButton = DUploadPictureButton;
